@@ -2,6 +2,24 @@
 L1L2Py (l1l2py) is a package to perform feature selection by means of l1l2
 regularization with double optimization.
 
+Main functions
+--------------
+l1l2py.minimal_model
+    Aims at finding the minimal (in terms of number of selected variables)
+    model within a k-fold cross validation loop.
+l1l2py.nested_models
+    Identifies the set of relevant lists of variables for increasing
+    correlation among them.
+l1l2py.model_selection
+    Runs the previous two stages sequentially.
+
+Available modules
+------------------
+l1l2py.algorithms
+    Efficiently implements the underlying optimization algorithms.
+l1l2py.tools
+    Implements miscellaneous useful tools.
+
 """
 ## This code is written by Salvatore Masecchia <salvatore.masecchia@unige.it>
 ## and Annalisa Barla <annalisa.barla@unige.it>
