@@ -28,6 +28,10 @@ elaboration of the data.
 __all__ = ['l1_bound', 'ridge_regression', 'l1l2_regularization', 'l1l2_path']
 
 import numpy as np
+try:
+    from scipy import linalg as la
+except ImportError:
+    from numpy import linalg as la
 
 def l1_bound(data, labels):
     r"""Estimation of an useful maximum bound for the `l1` penalty term.
@@ -64,8 +68,9 @@ def l1_bound(data, labels):
     >>> tau_max = l1l2py.algorithms.l1_bound(X, Y)
     >>> l1l2py.algorithms.l1l2_regularization(X, Y, 0.0, tau_max).T
     array([[ 0.,  0.,  0.]])
-    >>> l1l2py.algorithms.l1l2_regularization(X, Y, 0.0, tau_max - 1e-5).T
-    array([[  0.00000000e+00,   3.45622120e-06,   0.00000000e+00]])
+    >>> beta = l1l2py.algorithms.l1l2_regularization(X, Y, 0.0, tau_max - 1e-5)
+    >>> len(numpy.flatnonzero(beta))
+    1
 
     """
     n = data.shape[0]
@@ -100,11 +105,10 @@ def ridge_regression(data, labels, mu=0.0):
     >>> X = numpy.array([[0.1, 1.1, 0.3], [0.2, 1.2, 1.6], [0.3, 1.3, -0.6]])
     >>> beta = numpy.array([0.1, 0.1, 0.0])
     >>> Y = numpy.dot(X, beta)
-    >>> l1l2py.algorithms.ridge_regression(X, Y, 1e3).T
-    array([[  2.92871765e-05,   1.69054825e-04,   5.45274610e-05]])
-    >>> beta_ls = l1l2py.algorithms.ridge_regression(X, Y).T
-    >>> numpy.allclose(beta, beta_ls.squeeze())
-    True
+    >>> beta = l1l2py.algorithms.ridge_regression(X, Y, 1e3).T
+    >>> len(numpy.flatnonzero(beta))
+    3
+
     """
     n, p = data.shape
 
@@ -112,14 +116,14 @@ def ridge_regression(data, labels, mu=0.0):
         tmp = np.dot(data, data.T)
         if mu:
             tmp += mu*n*np.eye(n)
-        tmp = np.linalg.pinv(tmp)
+        tmp = la.pinv(tmp)
 
         return np.dot(np.dot(data.T, tmp), labels.reshape(-1, 1))
     else:
         tmp = np.dot(data.T, data)
         if mu:
             tmp += mu*n*np.eye(p)
-        tmp = np.linalg.pinv(tmp)
+        tmp = la.pinv(tmp)
 
         return np.dot(tmp, np.dot(data.T, labels.reshape(-1, 1)))
 
@@ -199,7 +203,7 @@ def l1l2_path(data, labels, mu, tau_range, beta=None, kmax=1e5,
 
 def l1l2_regularization(data, labels, mu, tau, beta=None, kmax=1e5,
                         tolerance=1e-5, return_iterations=False):
-    r"""Implementation of the Iterative Shrinkage-Thresholding Algorithm
+    r"""Implementation of the Fast Iterative Shrinkage-Thresholding Algorithm
     to solve a least squares problem with `l1l2` penalty.
 
     It solves the `l1l2` regularization problem with parameter ``mu`` on the
@@ -239,13 +243,12 @@ def l1l2_regularization(data, labels, mu, tau, beta=None, kmax=1e5,
     >>> X = numpy.array([[0.1, 1.1, 0.3], [0.2, 1.2, 1.6], [0.3, 1.3, -0.6]])
     >>> beta = numpy.array([0.1, 0.1, 0.0])
     >>> Y = numpy.dot(X, beta)
-    >>> l1l2py.algorithms.l1l2_regularization(X, Y, 0.1, 0.1).T
-    array([[ 0.        ,  0.07715517,  0.        ]])
-    >>> beta_ls = l1l2py.algorithms.l1l2_regularization(X, Y, 0.0, 0.0).T
-    >>> numpy.allclose(beta, beta_ls.squeeze())
-    True
+    >>> beta = l1l2py.algorithms.l1l2_regularization(X, Y, 0.1, 0.1)
+    >>> len(numpy.flatnonzero(beta))
+    1
 
     """
+    import math
     n = data.shape[0]
 
     # Useful quantities
@@ -255,24 +258,37 @@ def l1l2_regularization(data, labels, mu, tau, beta=None, kmax=1e5,
     XT = data.T / (n * sigma)
     XTY = np.dot(XT, labels.reshape(-1, 1))
 
+    # beta starts from 0 and we assume also that the previous value is 0
     if beta is None:
         beta = np.zeros_like(XTY)
     else:
         beta = beta.reshape(-1, 1)
+    beta_prev = beta
 
-    k, kmin = 0, 100
+    # Auxiliary beta (FISTA implementation), starts from 0
+    aux_beta = beta
+    t, t_next = 1., None     # t values initializations
+
+    k, kmin = 0, 10
     th, difference = -np.inf, np.inf
     while k < kmin or ((difference > th).any() and k < kmax):
         k += 1
 
-        value = beta +  XTY - np.dot(XT, np.dot(data, beta))
-        beta_next = _soft_thresholding(value, tau_s) / (1.0 + mu_s)
+        # New solution
+        value = (1.0 - mu_s)*aux_beta + XTY - np.dot(XT, np.dot(data, aux_beta))
+        beta = _soft_thresholding(value, tau_s)
+
+        # New auxiliary beta (FISTA)
+        t_next = 0.5 * (1.0 + math.sqrt(1.0 + 4.0 * t*t))
+        difference = (beta - beta_prev)
+        aux_beta = beta + ((t - 1.0)/t_next)*difference
 
         # Convergence values
-        difference = np.abs(beta_next - beta)
-        th = np.abs(beta) * (tolerance / k)
+        difference = np.abs(difference)
+        th = np.abs(beta) * (tolerance / math.sqrt(k))
 
-        beta = beta_next
+        # Values update
+        beta_prev, t = beta, t_next
 
     if return_iterations:
         return beta, k
@@ -284,13 +300,11 @@ def _sigma(matrix, mu):
 
     if p > n:
         tmp = np.dot(matrix, matrix.T)
-        num = np.linalg.eigvalsh(tmp).max()
     else:
         tmp = np.dot(matrix.T, matrix)
-        evals = np.linalg.eigvalsh(tmp)
-        num = evals.max() + evals.min()
 
-    return (num/(2.*n)) + mu
+    return (la.eigvalsh(tmp).max()/n) + mu
+
 
 def _soft_thresholding(x, th):
     return np.sign(x) * np.maximum(0, np.abs(x) - th/2.0)
